@@ -1,7 +1,7 @@
 
 #   IGraph R package
-#   Copyright (C) 2006  Gabor Csardi <csardi@rmki.kfki.hu>
-#   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+#   Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
+#   334 Harvard street, Cambridge, MA 02139 USA
 #   
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -33,8 +33,9 @@ cliques <- function(graph, min=NULL, max=NULL) {
   }
 
   on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_cliques", graph, as.numeric(min), as.numeric(max),
-        PACKAGE="igraph")
+  res <- .Call("R_igraph_cliques", graph, as.numeric(min), as.numeric(max),
+               PACKAGE="igraph")
+  lapply(res, function(x) x+1)
 }
 
 largest.cliques <- function(graph) {
@@ -43,18 +44,24 @@ largest.cliques <- function(graph) {
   }
 
   on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_largest_cliques", graph,
-        PACKAGE="igraph")
+  res <- .Call("R_igraph_largest_cliques", graph,
+               PACKAGE="igraph")
+  lapply(res, function(x) x+1)  
 }
 
-maximal.cliques <- function(graph) {
+maximal.cliques <- function(graph, min=NULL, max=NULL) {
   if (!is.igraph(graph)) {
     stop("Not a graph object");
   }
 
+  if (is.null(min)) { min <- 0 }
+  if (is.null(max)) { max <- 0 }
+
   on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_maximal_cliques", graph,
-        PACKAGE="igraph")
+  res <- .Call("R_igraph_maximal_cliques", graph, as.numeric(min),
+               as.numeric(max),
+               PACKAGE="igraph")
+  lapply(res, function(x) x+1)
 }
 
 clique.number <- function(graph) {
@@ -81,9 +88,10 @@ independent.vertex.sets <- function(graph, min=NULL, max=NULL) {
   }
   
   on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_independent_vertex_sets", graph, as.numeric(min),
-        as.numeric(max),
-        PACKAGE="igraph")
+  res <- .Call("R_igraph_independent_vertex_sets", graph, as.numeric(min),
+               as.numeric(max),
+               PACKAGE="igraph")
+  lapply(res, function(x) x+1)
 }
 
 largest.independent.vertex.sets <- function(graph) {
@@ -92,8 +100,9 @@ largest.independent.vertex.sets <- function(graph) {
   }
 
   on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_largest_independent_vertex_sets", graph,
-        PACKAGE="igraph")
+  res <- .Call("R_igraph_largest_independent_vertex_sets", graph,
+               PACKAGE="igraph")
+  lapply(res, function(x) x+1)
 }
 
 maximal.independent.vertex.sets <- function(graph) {
@@ -102,8 +111,9 @@ maximal.independent.vertex.sets <- function(graph) {
   }
 
   on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_maximal_independent_vertex_sets", graph,
-        PACKAGE="igraph")
+  res <- .Call("R_igraph_maximal_independent_vertex_sets", graph,
+               PACKAGE="igraph")
+  lapply(res, function(x) x+1)
 }
 
 independence.number <- function(graph) {
