@@ -1,7 +1,7 @@
 
 #   IGraph R package
-#   Copyright (C) 2005  Gabor Csardi <csardi@rmki.kfki.hu>
-#   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+#   Copyright (C) 2005-2012  Gabor Csardi <csardi.gabor@gmail.com>
+#   334 Harvard street, Cambridge, MA 02139 USA
 #   
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -20,21 +20,16 @@
 #
 ###################################################################
 
+.onAttach <- function(library, pkg) {
+    ## we can't do this in .onLoad
+    unlockBinding(".igraph.pars", asNamespace("igraph"))
+    unlockBinding(".igraph.pb", asNamespace("igraph"))
+    invisible()
+}
+
 .onLoad <- function(libname, pkgname) {
   library.dynam("igraph", pkgname, libname, local=FALSE);
-
-  ########################
-  # Set default parameters
-  ########################
-
-  # printing attributes
-  igraph.par("print.graph.attributes", FALSE)
-  igraph.par("print.vertex.attributes", FALSE)
-  igraph.par("print.edge.attributes", FALSE)
-
-  # verbosity, progress bars mainly
-  igraph.par("verbose", FALSE)
-  
+  .Call("R_igraph_init", FALSE, FALSE, PACKAGE="igraph")
 }
 
 .onUnload <- function(libpath) {
