@@ -17,12 +17,14 @@ Copyright (C) 2003-2006 Tommi Junttila
 
 /* FSF address fixed in the above notice on 1 Oct 2009 by Tamas Nepusz */
 
-#include <assert.h>
+#include <cassert>
 #include <vector>
 #include <list>
 #include "bliss_graph.hh"
 #include "bliss_partition.hh"
 
+using namespace std;
+
 namespace igraph {
 
 static const bool should_not_happen = false;
@@ -255,42 +257,6 @@ Cell *Partition::aux_split_in_two(Cell * const cell,
   return new_cell;
 } 
 
-
-void Partition::print(FILE *fp)
-{
-  const char *cell_sep = "";
-  fprintf(fp, "[");
-  for(Cell *cell = first_cell; cell; cell = cell->next)
-    {
-      fprintf(fp, "%s", cell_sep);
-      cell_sep = ",";
-      /* Print cell */
-      const char *elem_sep = "";
-      fprintf(fp, "{");
-      for(unsigned int i = 0; i < cell->length; i++)
-	{
-	  fprintf(fp, "%s%u", elem_sep, elements[cell->first + i]);
-	  elem_sep = ",";
-	}
-      fprintf(fp, "}");
-    }
-  fprintf(fp, "]");
-}
-
-
-void Partition::print_signature(FILE *fp)
-{
-  const char *cell_sep = "";
-  fprintf(fp, "[");
-  for(Cell *cell = first_cell; cell; cell = cell->next)
-    {
-      fprintf(fp, "%s%u", cell_sep, cell->length);
-      cell_sep = ",";
-    }
-  fprintf(fp, "]");
-}
-
-
 void Partition::add_in_splitting_queue(Cell * const cell)
 {
   static const unsigned int smallish_cell_threshold = 1;
@@ -462,7 +428,7 @@ Cell *Partition::sort_and_split_cell1(Cell *cell)
 /*
  * Tables and a subroutine for distribution count sorting
  */
-static unsigned int count[256] = {
+static IGRAPH_THREAD_LOCAL unsigned int count[256] = {
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
@@ -472,7 +438,7 @@ static unsigned int count[256] = {
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 };
-static unsigned int start[256];
+static IGRAPH_THREAD_LOCAL unsigned int start[256];
 
 /*
  * Build start array so that start[0] = 0 and start[i+1] = start[i] + count[i]
